@@ -1,20 +1,26 @@
-
-=head1 Pg::BulkCopy.pm and pg_bulkcopy.pl Version 0.16
-
-=cut
-
-
 use warnings;
 use strict;
 use Moose;
-use feature ":5.10" ;
+use 5.012 ;
 use Time::Piece ;
 use DBI ;
 use File::Copy ;
-use FileHandle; 
+use Log::Handler ;
+
+=pod
+
+=head1 NAME
+
+Pg::BulkCopy - Bulk Data Load/ Dump for Postgres.
+
+=head1 VERSION
+
+Version 0.21
+
+=cut
 
 package Pg::BulkCopy;
-our $VERSION = '0.16' ;
+our $VERSION = '0.21' ;
 use Moose ;
 	has 'dbistring' => ( isa => 'Str', is => 'rw', required => 1, ) ;
 	has 'filename' => ( isa => 'Str', is => 'rw', required => 1, ) ;	
@@ -72,8 +78,11 @@ sub BUILD {
 # The performance benefit of Lazy buffering should be trivial compared to the consequences having the log truncate
 # prematurely if the program exits unexpectedly. It created problems during testing where tests needed access to the logs
 # but the BulkCopy object had not been destroyed and had not flushed the buffer.
-		$self->{'EFH'} = FileHandle->new( ">>$self->{'errorlog'}" or die " can\'t open $self->{'errorlog'}\n" ) ;
-		$self->{'EFH'}->autoflush(1) ;
+		open ( $self->{'EFH'} , '>>', $self->{'errorlog'} ) or die " can\'t open $self->{'errorlog'}\n"  ;
+		select( $self->{'EFH'} );
+		$| = 1;
+	#	$self->{'EFH'} = FileHandle->new( ">>$self->{'errorlog'}" or die " can\'t open $self->{'errorlog'}\n" ) ;
+	#	eval { $self->{'EFH'}->autoflush(1) } ; say "Autoflush @_\ n\n" ;
 		}
  	my $t = localtime ;
  	$self->_DBG( 1, "Debug Logging to $self->{'errorlog'} $t" ) ;
@@ -192,7 +201,7 @@ sub LOAD {
 			my $DBHE = $DBH->errstr ;
 			# These chars might end up next to line or the numer we seek.
 			# translate turns them to spaces so they don't interfere.
-			$DBHE =~ tr/\:\,/  /d ;
+			$DBHE =~ tr/\:\,/  / ;
 			$self->_DBG( 2, "DBI Error Encountered. Attempting to identify and reject bad record.\n", $DBHE ) ;
 			my @ears = split /\s/, $DBHE  ;
 			while ( @ears ) {
@@ -269,7 +278,7 @@ $self->_DBG( 3, "BatchCount $batchcount, Iterator $iterator") ;
 $self->_DBG( 2, "Return string Set", $returnstring ) ;
 					$returnstatus  = -1 ; 
 					$finished = 1 ;
-					$self->TRUNC() ;
+#					$self->TRUNC() ;
 					}
 				else { $ReWrite->( $loaded ) ; }
 				} ; 
@@ -283,12 +292,18 @@ $self->_DBG( 2, "Return Value: $returnstatus" ) ;
 	return ( $returnstatus ) ;
 }	 #LOAD
 
+=head1 IMPORTANT NOTE on INSTALLATION
+
+If you are installing this through cpan please be aware that you will not be running any tests. 
+The tests are in t2 and not run by any of the CPAN clients. To run the tests you need to download 
+the test data L<http://www.cpan.org/authors/id/B/BR/BRAINBUZ/Pg-BulkCopyTest-0.16.tar.gz>. Pg::BulkCopy requires postgres to be installed on the same machine
+although it uses dbi it requires a shared directory that postgres and it can both write to. If you 
+do not want to setup and run the tests it is recommended that you at least perform a validation trial
+
 =head1 pg bulkCopy.pl
 
 The utility script pg_BulkCopy.pl was written to provide postgreSQL with a convient bulk loading utility. The script is implemented as a wrapper and a module (pg_BulkCopy.pl) so that other programmers may easily incorporate the two useful methods LOAD and DUMP directly into other perl scripts. 
 
-The advantage of this script over other scripts that have been written for this purpose is that if you can connect to and perform insert and delete operations on your database through the standard DBI interface (and it shouldn't matter which of the several postgres driver's you are using), pg_BulkCopy should just work.
-
 The DUMP Method invokes postgres' COPY TO command, and does nothing useful in addition except copying the dump from the temp directory (because postgres may not have permission on where you want the file). You can choose Tab Delimited Text or CSV with whatever delimiter you want and a Null string of your choice.
 
 The LOAD Method is much more interesting, it breaks the load file into chunks of 10000 (configurable) records in the temp directory and tries to COPY FROM, if it fails, it parses the error message for the line number, then it removes the failed line to a rejects file and tries again. As with DUMP you can select the options supported by the postgres COPY command, you can also set a limit on bad records (default is 10).
@@ -348,7 +363,7 @@ Command Line to load values from bulkcopy.conf but provide or override some valu
 
 =head3 dbistr, dbiuser, dbipass
 
-These are the parameters needed to establish a dbi connection. Please refer to the documentation establishing a database connection with dbi.
+These are the parameters needed to establish a dbi connection. 
 
 =head3 filename, table
 
@@ -362,7 +377,7 @@ Boolean values of 0 or 1. An B<iscsv> value of 1 indicates the file is csv. The
 
 B<workingdir> is where the file, reject and log files will be written, unless the full path/filename is specified it is also expected to find/write the file for the operation here. B<errorlog> is the name of a file to write information about problems to, this will default to <filename>.log. 
 
-B<tempdir> where the temporary working files will be written to. tempdir defaults to /tmp. Do not overlook tempdir the user executing the script and the uid that postgres is running under must have rw permissions here and the default creation mask must permit access to each other's newly created files! 
+B<tempdir> where the temporary working files will be written to. tempdir defaults to /tmp. Do not overlook tempdir, the user executing the script and the uid that postgres is running under must have rw permissions here and the default creation mask must permit access to each other's newly created files! 
 
 =head3 batchsize, maxerrors, debug
 
@@ -375,7 +390,7 @@ B<debug> can disable or increase the amount of error logging done. 0 disables er
 
 =head1 Module Pg::BulkCopy 
 
-All methods used by pg_BulkCopy.pl are provided by Pg::BulkCopy. The method names follow the convention of explicitely defined methods in caps and methods provided by Moose in lowercase.
+All methods used by pg_BulkCopy.pl are provided by Pg::BulkCopy. The method names follow the convention of explicitely defined methods in caps and methods created by Moose in lowercase.
 
 =head2 Systems Supported
 
@@ -471,7 +486,7 @@ To deal proactively with permissions issues I recommend the following steps. Che
 
 =head2 Other Issues
 
-There is currently an issue I haven't resolved with a quoted csv input test file. The next features I expect to work on involve supporting csv headers and field reordering, which will also make the feature available for tsv files.
+There is currently an issue I haven't resolved with a quoted csv input test file. The next features I expect to work on involve supporting csv headers and field reordering, which will also make the feature available for tsv files. 
 
 =head1 Options
 
@@ -481,7 +496,7 @@ CSV Headers are not supported yet, you'll need to chop them off yourself. Field
 
 =head1 Testing
 
-To properly test the module and script it is necessary to have an available configured database. So that the bundle can be installed silently through a cpan utility session very few tests are run during installation. Proper testing must be done manually. Due to the size of the test data it has been removed to a seperate archive, Pg-BulkCopyTest which must be downloaded seperately from cpan. Normally the contents would be restored to the tdata directory.
+To properly test the module and script it is necessary to have an available configured database. So that the bundle can be installed silently through a cpan utility session very few tests are run during installation. Proper testing must be done manually. Due to the size of the test data it has been removed to a seperate archive, Pg-BulkCopyTest which must be downloaded seperately from cpan. Normally the contents would be restored to the tdata directory.  
 
 =head2 Create and connect to the database
 
@@ -494,8 +509,6 @@ Edit the file t2/test.conf. You will need to provide the necessary dsn values fo
 
 If necessary modify harness.sh from the distribution directory as appropriate and execute it to run the tests.
 
-
-
 =head1 AUTHOR
 
 John Karr, C<< <brainbuz at brainbuz.org> >>
@@ -503,8 +516,7 @@ John Karr, C<< <brainbuz at brainbuz.org> >>
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-pg-bulkcopy at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Pg-BulkCopy>.  I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
+the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Pg-BulkCopy>.  I will be notified, and then you'll automatically be notified of progress on your bug as I make changes.
 
 
 
@@ -544,7 +556,7 @@ L<http://search.cpan.org/dist/Pg-BulkCopy/>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010 John Karr.
+Copyright 2010,2012 John Karr.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
